git.delta.rocks / unique-network / refs/commits / 176cbff9e8d3

difftreelog

Merge branch 'develop' into tests/eth-helpers

Max Andreev2022-12-16parents: #66caf45 #0a1b898.patch.diff
in: master

21 files changed

modifiedpallets/common/src/erc.rsdiffbeforeafterboth
--- a/pallets/common/src/erc.rs
+++ b/pallets/common/src/erc.rs
@@ -37,6 +37,7 @@
 	Pallet, CollectionHandle, Config, CollectionProperties, SelfWeightOf,
 	eth::{
 		EthCrossAccount, convert_cross_account_to_uint256, CollectionPermissions as EvmPermissions,
+		CollectionLimits as EvmCollectionLimits,
 	},
 	weights::WeightInfo,
 };
@@ -304,6 +305,99 @@
 		Ok(result)
 	}
 
+	/// Get current collection limits.
+	///
+	/// @return Array of tuples (byte, bool, uint256) with limits and their values. Order of limits:
+	/// 	"accountTokenOwnershipLimit",
+	/// 	"sponsoredDataSize",
+	/// 	"sponsoredDataRateLimit",
+	/// 	"tokenLimit",
+	/// 	"sponsorTransferTimeout",
+	/// 	"sponsorApproveTimeout"
+	///  	"ownerCanTransfer",
+	/// 	"ownerCanDestroy",
+	/// 	"transfersEnabled"
+	/// Return `false` if a limit not set.
+	fn collection_limits(&self) -> Result<Vec<(EvmCollectionLimits, bool, uint256)>> {
+		let convert_value_limit = |limit: EvmCollectionLimits,
+		                           value: Option<u32>|
+		 -> (EvmCollectionLimits, bool, uint256) {
+			value
+				.map(|v| (limit, true, v.into()))
+				.unwrap_or((limit, false, Default::default()))
+		};
+
+		let convert_bool_limit = |limit: EvmCollectionLimits,
+		                          value: Option<bool>|
+		 -> (EvmCollectionLimits, bool, uint256) {
+			value
+				.map(|v| {
+					(
+						limit,
+						true,
+						if v {
+							uint256::from(1)
+						} else {
+							Default::default()
+						},
+					)
+				})
+				.unwrap_or((limit, false, Default::default()))
+		};
+
+		let limits = &self.collection.limits;
+
+		Ok(vec![
+			convert_value_limit(
+				EvmCollectionLimits::AccountTokenOwnership,
+				limits.account_token_ownership_limit,
+			),
+			convert_value_limit(
+				EvmCollectionLimits::SponsoredDataSize,
+				limits.sponsored_data_size,
+			),
+			limits
+				.sponsored_data_rate_limit
+				.and_then(|limit| {
+					if let SponsoringRateLimit::Blocks(blocks) = limit {
+						Some((
+							EvmCollectionLimits::SponsoredDataRateLimit,
+							true,
+							blocks.into(),
+						))
+					} else {
+						None
+					}
+				})
+				.unwrap_or((
+					EvmCollectionLimits::SponsoredDataRateLimit,
+					false,
+					Default::default(),
+				)),
+			convert_value_limit(EvmCollectionLimits::TokenLimit, limits.token_limit),
+			convert_value_limit(
+				EvmCollectionLimits::SponsorTransferTimeout,
+				limits.sponsor_transfer_timeout,
+			),
+			convert_value_limit(
+				EvmCollectionLimits::SponsorApproveTimeout,
+				limits.sponsor_approve_timeout,
+			),
+			convert_bool_limit(
+				EvmCollectionLimits::OwnerCanTransfer,
+				limits.owner_can_transfer,
+			),
+			convert_bool_limit(
+				EvmCollectionLimits::OwnerCanDestroy,
+				limits.owner_can_destroy,
+			),
+			convert_bool_limit(
+				EvmCollectionLimits::TransferEnabled,
+				limits.transfers_enabled,
+			),
+		])
+	}
+
 	/// Set limits for the collection.
 	/// @dev Throws error if limit not found.
 	/// @param limit Name of the limit. Valid names:
@@ -316,11 +410,22 @@
 	///  	"ownerCanTransfer",
 	/// 	"ownerCanDestroy",
 	/// 	"transfersEnabled"
+	/// @param status enable\disable limit. Works only with `true`.
 	/// @param value Value of the limit.
 	#[solidity(rename_selector = "setCollectionLimit")]
-	fn set_int_limit(&mut self, caller: caller, limit: string, value: uint256) -> Result<void> {
+	fn set_collection_limit(
+		&mut self,
+		caller: caller,
+		limit: EvmCollectionLimits,
+		status: bool,
+		value: uint256,
+	) -> Result<void> {
 		self.consume_store_reads_and_writes(1, 1)?;
 
+		if !status {
+			return Err(Error::Revert("user can't disable limits".into()));
+		}
+
 		let value = value
 			.try_into()
 			.map_err(|_| Error::Revert(format!("can't convert value to u32 \"{}\"", value)))?;
@@ -338,35 +443,35 @@
 
 		let mut limits = self.limits.clone();
 
-		match limit.as_str() {
-			"accountTokenOwnershipLimit" => {
+		match limit {
+			EvmCollectionLimits::AccountTokenOwnership => {
 				limits.account_token_ownership_limit = Some(value);
 			}
-			"sponsoredDataSize" => {
+			EvmCollectionLimits::SponsoredDataSize => {
 				limits.sponsored_data_size = Some(value);
 			}
-			"sponsoredDataRateLimit" => {
+			EvmCollectionLimits::SponsoredDataRateLimit => {
 				limits.sponsored_data_rate_limit = Some(SponsoringRateLimit::Blocks(value));
 			}
-			"tokenLimit" => {
+			EvmCollectionLimits::TokenLimit => {
 				limits.token_limit = Some(value);
 			}
-			"sponsorTransferTimeout" => {
+			EvmCollectionLimits::SponsorTransferTimeout => {
 				limits.sponsor_transfer_timeout = Some(value);
 			}
-			"sponsorApproveTimeout" => {
+			EvmCollectionLimits::SponsorApproveTimeout => {
 				limits.sponsor_approve_timeout = Some(value);
 			}
-			"ownerCanTransfer" => {
+			EvmCollectionLimits::OwnerCanTransfer => {
 				limits.owner_can_transfer = Some(convert_value_to_bool()?);
 			}
-			"ownerCanDestroy" => {
+			EvmCollectionLimits::OwnerCanDestroy => {
 				limits.owner_can_destroy = Some(convert_value_to_bool()?);
 			}
-			"transfersEnabled" => {
+			EvmCollectionLimits::TransferEnabled => {
 				limits.transfers_enabled = Some(convert_value_to_bool()?);
 			}
-			_ => return Err(Error::Revert(format!("unknown limit \"{}\"", limit))),
+			_ => return Err(Error::Revert(format!("unknown limit \"{:?}\"", limit))),
 		}
 
 		let caller = T::CrossAccountId::from_eth(caller);
modifiedpallets/common/src/eth.rsdiffbeforeafterboth
--- a/pallets/common/src/eth.rs
+++ b/pallets/common/src/eth.rs
@@ -155,6 +155,31 @@
 		}
 	}
 }
+
+/// [`CollectionLimits`](up_data_structs::CollectionLimits) representation for EVM.
+#[derive(Debug, Default, Clone, Copy, AbiCoder)]
+#[repr(u8)]
+pub enum CollectionLimits {
+	/// How many tokens can a user have on one account.
+	#[default]
+	AccountTokenOwnership,
+	/// How many bytes of data are available for sponsorship.
+	SponsoredDataSize,
+	/// In any case, chain default: [`SponsoringRateLimit::SponsoringDisabled`]
+	SponsoredDataRateLimit,
+	/// How many tokens can be mined into this collection.
+	TokenLimit,
+	/// Timeouts for transfer sponsoring.
+	SponsorTransferTimeout,
+	/// Timeout for sponsoring an approval in passed blocks.
+	SponsorApproveTimeout,
+	/// Whether the collection owner of the collection can send tokens (which belong to other users).
+	OwnerCanTransfer,
+	/// Can the collection owner burn other people's tokens.
+	OwnerCanDestroy,
+	/// Is it possible to send tokens from this collection between users.
+	TransferEnabled,
+}
 #[derive(Default, Debug, Clone, Copy, AbiCoder)]
 #[repr(u8)]
 pub enum CollectionPermissions {
modifiedpallets/fungible/src/stubs/UniqueFungible.rawdiffbeforeafterboth

binary blob — no preview

modifiedpallets/fungible/src/stubs/UniqueFungible.soldiffbeforeafterboth
before · pallets/fungible/src/stubs/UniqueFungible.sol
1// SPDX-License-Identifier: OTHER2// This code is automatically generated34pragma solidity >=0.8.0 <0.9.0;56/// @dev common stubs holder7contract Dummy {8	uint8 dummy;9	string stub_error = "this contract is implemented in native";10}1112contract ERC165 is Dummy {13	function supportsInterface(bytes4 interfaceID) external view returns (bool) {14		require(false, stub_error);15		interfaceID;16		return true;17	}18}1920/// @title A contract that allows you to work with collections.21/// @dev the ERC-165 identifier for this interface is 0xb5e1747f22contract Collection is Dummy, ERC165 {23	// /// Set collection property.24	// ///25	// /// @param key Property key.26	// /// @param value Propery value.27	// /// @dev EVM selector for this function is: 0x2f073f66,28	// ///  or in textual repr: setCollectionProperty(string,bytes)29	// function setCollectionProperty(string memory key, bytes memory value) public {30	// 	require(false, stub_error);31	// 	key;32	// 	value;33	// 	dummy = 0;34	// }3536	/// Set collection properties.37	///38	/// @param properties Vector of properties key/value pair.39	/// @dev EVM selector for this function is: 0x50b26b2a,40	///  or in textual repr: setCollectionProperties((string,bytes)[])41	function setCollectionProperties(Property[] memory properties) public {42		require(false, stub_error);43		properties;44		dummy = 0;45	}4647	// /// Delete collection property.48	// ///49	// /// @param key Property key.50	// /// @dev EVM selector for this function is: 0x7b7debce,51	// ///  or in textual repr: deleteCollectionProperty(string)52	// function deleteCollectionProperty(string memory key) public {53	// 	require(false, stub_error);54	// 	key;55	// 	dummy = 0;56	// }5758	/// Delete collection properties.59	///60	/// @param keys Properties keys.61	/// @dev EVM selector for this function is: 0xee206ee3,62	///  or in textual repr: deleteCollectionProperties(string[])63	function deleteCollectionProperties(string[] memory keys) public {64		require(false, stub_error);65		keys;66		dummy = 0;67	}6869	/// Get collection property.70	///71	/// @dev Throws error if key not found.72	///73	/// @param key Property key.74	/// @return bytes The property corresponding to the key.75	/// @dev EVM selector for this function is: 0xcf24fd6d,76	///  or in textual repr: collectionProperty(string)77	function collectionProperty(string memory key) public view returns (bytes memory) {78		require(false, stub_error);79		key;80		dummy;81		return hex"";82	}8384	/// Get collection properties.85	///86	/// @param keys Properties keys. Empty keys for all propertyes.87	/// @return Vector of properties key/value pairs.88	/// @dev EVM selector for this function is: 0x285fb8e6,89	///  or in textual repr: collectionProperties(string[])90	function collectionProperties(string[] memory keys) public view returns (Property[] memory) {91		require(false, stub_error);92		keys;93		dummy;94		return new Property[](0);95	}9697	// /// Set the sponsor of the collection.98	// ///99	// /// @dev In order for sponsorship to work, it must be confirmed on behalf of the sponsor.100	// ///101	// /// @param sponsor Address of the sponsor from whose account funds will be debited for operations with the contract.102	// /// @dev EVM selector for this function is: 0x7623402e,103	// ///  or in textual repr: setCollectionSponsor(address)104	// function setCollectionSponsor(address sponsor) public {105	// 	require(false, stub_error);106	// 	sponsor;107	// 	dummy = 0;108	// }109110	/// Set the sponsor of the collection.111	///112	/// @dev In order for sponsorship to work, it must be confirmed on behalf of the sponsor.113	///114	/// @param sponsor Cross account address of the sponsor from whose account funds will be debited for operations with the contract.115	/// @dev EVM selector for this function is: 0x84a1d5a8,116	///  or in textual repr: setCollectionSponsorCross((address,uint256))117	function setCollectionSponsorCross(EthCrossAccount memory sponsor) public {118		require(false, stub_error);119		sponsor;120		dummy = 0;121	}122123	/// Whether there is a pending sponsor.124	/// @dev EVM selector for this function is: 0x058ac185,125	///  or in textual repr: hasCollectionPendingSponsor()126	function hasCollectionPendingSponsor() public view returns (bool) {127		require(false, stub_error);128		dummy;129		return false;130	}131132	/// Collection sponsorship confirmation.133	///134	/// @dev After setting the sponsor for the collection, it must be confirmed with this function.135	/// @dev EVM selector for this function is: 0x3c50e97a,136	///  or in textual repr: confirmCollectionSponsorship()137	function confirmCollectionSponsorship() public {138		require(false, stub_error);139		dummy = 0;140	}141142	/// Remove collection sponsor.143	/// @dev EVM selector for this function is: 0x6e0326a3,144	///  or in textual repr: removeCollectionSponsor()145	function removeCollectionSponsor() public {146		require(false, stub_error);147		dummy = 0;148	}149150	/// Get current sponsor.151	///152	/// @return Tuble with sponsor address and his substrate mirror. If there is no confirmed sponsor error "Contract has no sponsor" throw.153	/// @dev EVM selector for this function is: 0x6ec0a9f1,154	///  or in textual repr: collectionSponsor()155	function collectionSponsor() public view returns (Tuple8 memory) {156		require(false, stub_error);157		dummy;158		return Tuple8(0x0000000000000000000000000000000000000000, 0);159	}160161	/// Set limits for the collection.162	/// @dev Throws error if limit not found.163	/// @param limit Name of the limit. Valid names:164	/// 	"accountTokenOwnershipLimit",165	/// 	"sponsoredDataSize",166	/// 	"sponsoredDataRateLimit",167	/// 	"tokenLimit",168	/// 	"sponsorTransferTimeout",169	/// 	"sponsorApproveTimeout"170	///  	"ownerCanTransfer",171	/// 	"ownerCanDestroy",172	/// 	"transfersEnabled"173	/// @param value Value of the limit.174	/// @dev EVM selector for this function is: 0x4ad890a8,175	///  or in textual repr: setCollectionLimit(string,uint256)176	function setCollectionLimit(string memory limit, uint256 value) public {177		require(false, stub_error);178		limit;179		value;180		dummy = 0;181	}182183	/// Get contract address.184	/// @dev EVM selector for this function is: 0xf6b4dfb4,185	///  or in textual repr: contractAddress()186	function contractAddress() public view returns (address) {187		require(false, stub_error);188		dummy;189		return 0x0000000000000000000000000000000000000000;190	}191192	/// Add collection admin.193	/// @param newAdmin Cross account administrator address.194	/// @dev EVM selector for this function is: 0x859aa7d6,195	///  or in textual repr: addCollectionAdminCross((address,uint256))196	function addCollectionAdminCross(EthCrossAccount memory newAdmin) public {197		require(false, stub_error);198		newAdmin;199		dummy = 0;200	}201202	/// Remove collection admin.203	/// @param admin Cross account administrator address.204	/// @dev EVM selector for this function is: 0x6c0cd173,205	///  or in textual repr: removeCollectionAdminCross((address,uint256))206	function removeCollectionAdminCross(EthCrossAccount memory admin) public {207		require(false, stub_error);208		admin;209		dummy = 0;210	}211212	// /// Add collection admin.213	// /// @param newAdmin Address of the added administrator.214	// /// @dev EVM selector for this function is: 0x92e462c7,215	// ///  or in textual repr: addCollectionAdmin(address)216	// function addCollectionAdmin(address newAdmin) public {217	// 	require(false, stub_error);218	// 	newAdmin;219	// 	dummy = 0;220	// }221222	// /// Remove collection admin.223	// ///224	// /// @param admin Address of the removed administrator.225	// /// @dev EVM selector for this function is: 0xfafd7b42,226	// ///  or in textual repr: removeCollectionAdmin(address)227	// function removeCollectionAdmin(address admin) public {228	// 	require(false, stub_error);229	// 	admin;230	// 	dummy = 0;231	// }232233	/// Toggle accessibility of collection nesting.234	///235	/// @param enable If "true" degenerates to nesting: 'Owner' else to nesting: 'Disabled'236	/// @dev EVM selector for this function is: 0x112d4586,237	///  or in textual repr: setCollectionNesting(bool)238	function setCollectionNesting(bool enable) public {239		require(false, stub_error);240		enable;241		dummy = 0;242	}243244	/// Toggle accessibility of collection nesting.245	///246	/// @param enable If "true" degenerates to nesting: {OwnerRestricted: [1, 2, 3]} else to nesting: 'Disabled'247	/// @param collections Addresses of collections that will be available for nesting.248	/// @dev EVM selector for this function is: 0x64872396,249	///  or in textual repr: setCollectionNesting(bool,address[])250	function setCollectionNesting(bool enable, address[] memory collections) public {251		require(false, stub_error);252		enable;253		collections;254		dummy = 0;255	}256257	/// Returns nesting for a collection258	/// @dev EVM selector for this function is: 0x22d25bfe,259	///  or in textual repr: collectionNestingRestrictedCollectionIds()260	function collectionNestingRestrictedCollectionIds() public view returns (Tuple21 memory) {261		require(false, stub_error);262		dummy;263		return Tuple21(false, new uint256[](0));264	}265266	/// Returns permissions for a collection267	/// @dev EVM selector for this function is: 0x5b2eaf4b,268	///  or in textual repr: collectionNestingPermissions()269	function collectionNestingPermissions() public view returns (Tuple24[] memory) {270		require(false, stub_error);271		dummy;272		return new Tuple24[](0);273	}274275	/// Set the collection access method.276	/// @param mode Access mode277	/// 	0 for Normal278	/// 	1 for AllowList279	/// @dev EVM selector for this function is: 0x41835d4c,280	///  or in textual repr: setCollectionAccess(uint8)281	function setCollectionAccess(uint8 mode) public {282		require(false, stub_error);283		mode;284		dummy = 0;285	}286287	/// Checks that user allowed to operate with collection.288	///289	/// @param user User address to check.290	/// @dev EVM selector for this function is: 0x91b6df49,291	///  or in textual repr: allowlistedCross((address,uint256))292	function allowlistedCross(EthCrossAccount memory user) public view returns (bool) {293		require(false, stub_error);294		user;295		dummy;296		return false;297	}298299	// /// Add the user to the allowed list.300	// ///301	// /// @param user Address of a trusted user.302	// /// @dev EVM selector for this function is: 0x67844fe6,303	// ///  or in textual repr: addToCollectionAllowList(address)304	// function addToCollectionAllowList(address user) public {305	// 	require(false, stub_error);306	// 	user;307	// 	dummy = 0;308	// }309310	/// Add user to allowed list.311	///312	/// @param user User cross account address.313	/// @dev EVM selector for this function is: 0xa0184a3a,314	///  or in textual repr: addToCollectionAllowListCross((address,uint256))315	function addToCollectionAllowListCross(EthCrossAccount memory user) public {316		require(false, stub_error);317		user;318		dummy = 0;319	}320321	// /// Remove the user from the allowed list.322	// ///323	// /// @param user Address of a removed user.324	// /// @dev EVM selector for this function is: 0x85c51acb,325	// ///  or in textual repr: removeFromCollectionAllowList(address)326	// function removeFromCollectionAllowList(address user) public {327	// 	require(false, stub_error);328	// 	user;329	// 	dummy = 0;330	// }331332	/// Remove user from allowed list.333	///334	/// @param user User cross account address.335	/// @dev EVM selector for this function is: 0x09ba452a,336	///  or in textual repr: removeFromCollectionAllowListCross((address,uint256))337	function removeFromCollectionAllowListCross(EthCrossAccount memory user) public {338		require(false, stub_error);339		user;340		dummy = 0;341	}342343	/// Switch permission for minting.344	///345	/// @param mode Enable if "true".346	/// @dev EVM selector for this function is: 0x00018e84,347	///  or in textual repr: setCollectionMintMode(bool)348	function setCollectionMintMode(bool mode) public {349		require(false, stub_error);350		mode;351		dummy = 0;352	}353354	// /// Check that account is the owner or admin of the collection355	// ///356	// /// @param user account to verify357	// /// @return "true" if account is the owner or admin358	// /// @dev EVM selector for this function is: 0x9811b0c7,359	// ///  or in textual repr: isOwnerOrAdmin(address)360	// function isOwnerOrAdmin(address user) public view returns (bool) {361	// 	require(false, stub_error);362	// 	user;363	// 	dummy;364	// 	return false;365	// }366367	/// Check that account is the owner or admin of the collection368	///369	/// @param user User cross account to verify370	/// @return "true" if account is the owner or admin371	/// @dev EVM selector for this function is: 0x3e75a905,372	///  or in textual repr: isOwnerOrAdminCross((address,uint256))373	function isOwnerOrAdminCross(EthCrossAccount memory user) public view returns (bool) {374		require(false, stub_error);375		user;376		dummy;377		return false;378	}379380	/// Returns collection type381	///382	/// @return `Fungible` or `NFT` or `ReFungible`383	/// @dev EVM selector for this function is: 0xd34b55b8,384	///  or in textual repr: uniqueCollectionType()385	function uniqueCollectionType() public view returns (string memory) {386		require(false, stub_error);387		dummy;388		return "";389	}390391	/// Get collection owner.392	///393	/// @return Tuble with sponsor address and his substrate mirror.394	/// If address is canonical then substrate mirror is zero and vice versa.395	/// @dev EVM selector for this function is: 0xdf727d3b,396	///  or in textual repr: collectionOwner()397	function collectionOwner() public view returns (EthCrossAccount memory) {398		require(false, stub_error);399		dummy;400		return EthCrossAccount(0x0000000000000000000000000000000000000000, 0);401	}402403	// /// Changes collection owner to another account404	// ///405	// /// @dev Owner can be changed only by current owner406	// /// @param newOwner new owner account407	// /// @dev EVM selector for this function is: 0x4f53e226,408	// ///  or in textual repr: changeCollectionOwner(address)409	// function changeCollectionOwner(address newOwner) public {410	// 	require(false, stub_error);411	// 	newOwner;412	// 	dummy = 0;413	// }414415	/// Get collection administrators416	///417	/// @return Vector of tuples with admins address and his substrate mirror.418	/// If address is canonical then substrate mirror is zero and vice versa.419	/// @dev EVM selector for this function is: 0x5813216b,420	///  or in textual repr: collectionAdmins()421	function collectionAdmins() public view returns (EthCrossAccount[] memory) {422		require(false, stub_error);423		dummy;424		return new EthCrossAccount[](0);425	}426427	/// Changes collection owner to another account428	///429	/// @dev Owner can be changed only by current owner430	/// @param newOwner new owner cross account431	/// @dev EVM selector for this function is: 0x6496c497,432	///  or in textual repr: changeCollectionOwnerCross((address,uint256))433	function changeCollectionOwnerCross(EthCrossAccount memory newOwner) public {434		require(false, stub_error);435		newOwner;436		dummy = 0;437	}438}439440/// @dev Cross account struct441struct EthCrossAccount {442	address eth;443	uint256 sub;444}445446enum CollectionPermissions {447	CollectionAdmin,448	TokenOwner449}450451/// @dev anonymous struct452struct Tuple24 {453	CollectionPermissions field_0;454	bool field_1;455}456457/// @dev anonymous struct458struct Tuple21 {459	bool field_0;460	uint256[] field_1;461}462463/// @dev Property struct464struct Property {465	string key;466	bytes value;467}468469/// @dev the ERC-165 identifier for this interface is 0x5b7038cf470contract ERC20UniqueExtensions is Dummy, ERC165 {471	/// @notice A description for the collection.472	/// @dev EVM selector for this function is: 0x7284e416,473	///  or in textual repr: description()474	function description() public view returns (string memory) {475		require(false, stub_error);476		dummy;477		return "";478	}479480	/// @dev EVM selector for this function is: 0x0ecd0ab0,481	///  or in textual repr: approveCross((address,uint256),uint256)482	function approveCross(EthCrossAccount memory spender, uint256 amount) public returns (bool) {483		require(false, stub_error);484		spender;485		amount;486		dummy = 0;487		return false;488	}489490	// /// Burn tokens from account491	// /// @dev Function that burns an `amount` of the tokens of a given account,492	// /// deducting from the sender's allowance for said account.493	// /// @param from The account whose tokens will be burnt.494	// /// @param amount The amount that will be burnt.495	// /// @dev EVM selector for this function is: 0x79cc6790,496	// ///  or in textual repr: burnFrom(address,uint256)497	// function burnFrom(address from, uint256 amount) public returns (bool) {498	// 	require(false, stub_error);499	// 	from;500	// 	amount;501	// 	dummy = 0;502	// 	return false;503	// }504505	/// Burn tokens from account506	/// @dev Function that burns an `amount` of the tokens of a given account,507	/// deducting from the sender's allowance for said account.508	/// @param from The account whose tokens will be burnt.509	/// @param amount The amount that will be burnt.510	/// @dev EVM selector for this function is: 0xbb2f5a58,511	///  or in textual repr: burnFromCross((address,uint256),uint256)512	function burnFromCross(EthCrossAccount memory from, uint256 amount) public returns (bool) {513		require(false, stub_error);514		from;515		amount;516		dummy = 0;517		return false;518	}519520	/// Mint tokens for multiple accounts.521	/// @param amounts array of pairs of account address and amount522	/// @dev EVM selector for this function is: 0x1acf2d55,523	///  or in textual repr: mintBulk((address,uint256)[])524	function mintBulk(Tuple8[] memory amounts) public returns (bool) {525		require(false, stub_error);526		amounts;527		dummy = 0;528		return false;529	}530531	/// @dev EVM selector for this function is: 0x2ada85ff,532	///  or in textual repr: transferCross((address,uint256),uint256)533	function transferCross(EthCrossAccount memory to, uint256 amount) public returns (bool) {534		require(false, stub_error);535		to;536		amount;537		dummy = 0;538		return false;539	}540541	/// @dev EVM selector for this function is: 0xd5cf430b,542	///  or in textual repr: transferFromCross((address,uint256),(address,uint256),uint256)543	function transferFromCross(544		EthCrossAccount memory from,545		EthCrossAccount memory to,546		uint256 amount547	) public returns (bool) {548		require(false, stub_error);549		from;550		to;551		amount;552		dummy = 0;553		return false;554	}555}556557/// @dev anonymous struct558struct Tuple8 {559	address field_0;560	uint256 field_1;561}562563/// @dev the ERC-165 identifier for this interface is 0x40c10f19564contract ERC20Mintable is Dummy, ERC165 {565	/// Mint tokens for `to` account.566	/// @param to account that will receive minted tokens567	/// @param amount amount of tokens to mint568	/// @dev EVM selector for this function is: 0x40c10f19,569	///  or in textual repr: mint(address,uint256)570	function mint(address to, uint256 amount) public returns (bool) {571		require(false, stub_error);572		to;573		amount;574		dummy = 0;575		return false;576	}577}578579/// @dev inlined interface580contract ERC20Events {581	event Transfer(address indexed from, address indexed to, uint256 value);582	event Approval(address indexed owner, address indexed spender, uint256 value);583}584585/// @dev the ERC-165 identifier for this interface is 0x8cb847c4586contract ERC20 is Dummy, ERC165, ERC20Events {587	/// @dev EVM selector for this function is: 0x06fdde03,588	///  or in textual repr: name()589	function name() public view returns (string memory) {590		require(false, stub_error);591		dummy;592		return "";593	}594595	/// @dev EVM selector for this function is: 0x95d89b41,596	///  or in textual repr: symbol()597	function symbol() public view returns (string memory) {598		require(false, stub_error);599		dummy;600		return "";601	}602603	/// @dev EVM selector for this function is: 0x18160ddd,604	///  or in textual repr: totalSupply()605	function totalSupply() public view returns (uint256) {606		require(false, stub_error);607		dummy;608		return 0;609	}610611	/// @dev EVM selector for this function is: 0x313ce567,612	///  or in textual repr: decimals()613	function decimals() public view returns (uint8) {614		require(false, stub_error);615		dummy;616		return 0;617	}618619	/// @dev EVM selector for this function is: 0x70a08231,620	///  or in textual repr: balanceOf(address)621	function balanceOf(address owner) public view returns (uint256) {622		require(false, stub_error);623		owner;624		dummy;625		return 0;626	}627628	/// @dev EVM selector for this function is: 0xa9059cbb,629	///  or in textual repr: transfer(address,uint256)630	function transfer(address to, uint256 amount) public returns (bool) {631		require(false, stub_error);632		to;633		amount;634		dummy = 0;635		return false;636	}637638	/// @dev EVM selector for this function is: 0x23b872dd,639	///  or in textual repr: transferFrom(address,address,uint256)640	function transferFrom(641		address from,642		address to,643		uint256 amount644	) public returns (bool) {645		require(false, stub_error);646		from;647		to;648		amount;649		dummy = 0;650		return false;651	}652653	/// @dev EVM selector for this function is: 0x095ea7b3,654	///  or in textual repr: approve(address,uint256)655	function approve(address spender, uint256 amount) public returns (bool) {656		require(false, stub_error);657		spender;658		amount;659		dummy = 0;660		return false;661	}662663	/// @dev EVM selector for this function is: 0xdd62ed3e,664	///  or in textual repr: allowance(address,address)665	function allowance(address owner, address spender) public view returns (uint256) {666		require(false, stub_error);667		owner;668		spender;669		dummy;670		return 0;671	}672673	/// @notice Returns collection helper contract address674	/// @dev EVM selector for this function is: 0x1896cce6,675	///  or in textual repr: collectionHelperAddress()676	function collectionHelperAddress() public view returns (address) {677		require(false, stub_error);678		dummy;679		return 0x0000000000000000000000000000000000000000;680	}681}682683contract UniqueFungible is Dummy, ERC165, ERC20, ERC20Mintable, ERC20UniqueExtensions, Collection {}
modifiedpallets/nonfungible/src/stubs/UniqueNFT.rawdiffbeforeafterboth

binary blob — no preview

modifiedpallets/nonfungible/src/stubs/UniqueNFT.soldiffbeforeafterboth
--- a/tests/src/eth/abi/fungible.json
+++ b/tests/src/eth/abi/fungible.json
@@ -208,6 +208,28 @@
   },
   {
     "inputs": [],
+    "name": "collectionLimits",
+    "outputs": [
+      {
+        "components": [
+          {
+            "internalType": "enum CollectionLimits",
+            "name": "field_0",
+            "type": "uint8"
+          },
+          { "internalType": "bool", "name": "field_1", "type": "bool" },
+          { "internalType": "uint256", "name": "field_2", "type": "uint256" }
+        ],
+        "internalType": "struct Tuple20[]",
+        "name": "",
+        "type": "tuple[]"
+      }
+    ],
+    "stateMutability": "view",
+    "type": "function"
+  },
+  {
+    "inputs": [],
     "name": "collectionNestingPermissions",
     "outputs": [
       {
@@ -219,7 +241,7 @@
           },
           { "internalType": "bool", "name": "field_1", "type": "bool" }
         ],
-        "internalType": "struct Tuple24[]",
+        "internalType": "struct Tuple29[]",
         "name": "",
         "type": "tuple[]"
       }
@@ -240,7 +262,7 @@
             "type": "uint256[]"
           }
         ],
-        "internalType": "struct Tuple21",
+        "internalType": "struct Tuple26",
         "name": "",
         "type": "tuple"
       }
@@ -453,7 +475,12 @@
   },
   {
     "inputs": [
-      { "internalType": "string", "name": "limit", "type": "string" },
+      {
+        "internalType": "enum CollectionLimits",
+        "name": "limit",
+        "type": "uint8"
+      },
+      { "internalType": "bool", "name": "status", "type": "bool" },
       { "internalType": "uint256", "name": "value", "type": "uint256" }
     ],
     "name": "setCollectionLimit",
modifiedtests/src/eth/abi/nonFungible.jsondiffbeforeafterboth
--- a/tests/src/eth/abi/nonFungible.json
+++ b/tests/src/eth/abi/nonFungible.json
@@ -238,6 +238,28 @@
   },
   {
     "inputs": [],
+    "name": "collectionLimits",
+    "outputs": [
+      {
+        "components": [
+          {
+            "internalType": "enum CollectionLimits",
+            "name": "field_0",
+            "type": "uint8"
+          },
+          { "internalType": "bool", "name": "field_1", "type": "bool" },
+          { "internalType": "uint256", "name": "field_2", "type": "uint256" }
+        ],
+        "internalType": "struct Tuple33[]",
+        "name": "",
+        "type": "tuple[]"
+      }
+    ],
+    "stateMutability": "view",
+    "type": "function"
+  },
+  {
+    "inputs": [],
     "name": "collectionNestingPermissions",
     "outputs": [
       {
@@ -249,7 +271,7 @@
           },
           { "internalType": "bool", "name": "field_1", "type": "bool" }
         ],
-        "internalType": "struct Tuple37[]",
+        "internalType": "struct Tuple42[]",
         "name": "",
         "type": "tuple[]"
       }
@@ -270,7 +292,7 @@
             "type": "uint256[]"
           }
         ],
-        "internalType": "struct Tuple34",
+        "internalType": "struct Tuple39",
         "name": "",
         "type": "tuple"
       }
@@ -607,7 +629,12 @@
   },
   {
     "inputs": [
-      { "internalType": "string", "name": "limit", "type": "string" },
+      {
+        "internalType": "enum CollectionLimits",
+        "name": "limit",
+        "type": "uint8"
+      },
+      { "internalType": "bool", "name": "status", "type": "bool" },
       { "internalType": "uint256", "name": "value", "type": "uint256" }
     ],
     "name": "setCollectionLimit",
@@ -709,12 +736,12 @@
               },
               { "internalType": "bool", "name": "field_1", "type": "bool" }
             ],
-            "internalType": "struct Tuple46[]",
+            "internalType": "struct Tuple57[]",
             "name": "field_1",
             "type": "tuple[]"
           }
         ],
-        "internalType": "struct Tuple48[]",
+        "internalType": "struct Tuple59[]",
         "name": "permissions",
         "type": "tuple[]"
       }
@@ -775,12 +802,12 @@
               },
               { "internalType": "bool", "name": "field_1", "type": "bool" }
             ],
-            "internalType": "struct Tuple46[]",
+            "internalType": "struct Tuple57[]",
             "name": "field_1",
             "type": "tuple[]"
           }
         ],
-        "internalType": "struct Tuple48[]",
+        "internalType": "struct Tuple59[]",
         "name": "",
         "type": "tuple[]"
       }
modifiedtests/src/eth/abi/reFungible.jsondiffbeforeafterboth
--- a/tests/src/eth/abi/reFungible.json
+++ b/tests/src/eth/abi/reFungible.json
@@ -220,6 +220,28 @@
   },
   {
     "inputs": [],
+    "name": "collectionLimits",
+    "outputs": [
+      {
+        "components": [
+          {
+            "internalType": "enum CollectionLimits",
+            "name": "field_0",
+            "type": "uint8"
+          },
+          { "internalType": "bool", "name": "field_1", "type": "bool" },
+          { "internalType": "uint256", "name": "field_2", "type": "uint256" }
+        ],
+        "internalType": "struct Tuple32[]",
+        "name": "",
+        "type": "tuple[]"
+      }
+    ],
+    "stateMutability": "view",
+    "type": "function"
+  },
+  {
+    "inputs": [],
     "name": "collectionNestingPermissions",
     "outputs": [
       {
@@ -231,7 +253,7 @@
           },
           { "internalType": "bool", "name": "field_1", "type": "bool" }
         ],
-        "internalType": "struct Tuple36[]",
+        "internalType": "struct Tuple41[]",
         "name": "",
         "type": "tuple[]"
       }
@@ -252,7 +274,7 @@
             "type": "uint256[]"
           }
         ],
-        "internalType": "struct Tuple33",
+        "internalType": "struct Tuple38",
         "name": "",
         "type": "tuple"
       }
@@ -589,7 +611,12 @@
   },
   {
     "inputs": [
-      { "internalType": "string", "name": "limit", "type": "string" },
+      {
+        "internalType": "enum CollectionLimits",
+        "name": "limit",
+        "type": "uint8"
+      },
+      { "internalType": "bool", "name": "status", "type": "bool" },
       { "internalType": "uint256", "name": "value", "type": "uint256" }
     ],
     "name": "setCollectionLimit",
@@ -691,12 +718,12 @@
               },
               { "internalType": "bool", "name": "field_1", "type": "bool" }
             ],
-            "internalType": "struct Tuple51[]",
+            "internalType": "struct Tuple56[]",
             "name": "field_1",
             "type": "tuple[]"
           }
         ],
-        "internalType": "struct Tuple53[]",
+        "internalType": "struct Tuple58[]",
         "name": "permissions",
         "type": "tuple[]"
       }
@@ -766,12 +793,12 @@
               },
               { "internalType": "bool", "name": "field_1", "type": "bool" }
             ],
-            "internalType": "struct Tuple51[]",
+            "internalType": "struct Tuple56[]",
             "name": "field_1",
             "type": "tuple[]"
           }
         ],
-        "internalType": "struct Tuple53[]",
+        "internalType": "struct Tuple58[]",
         "name": "",
         "type": "tuple[]"
       }
modifiedtests/src/eth/api/UniqueFungible.soldiffbeforeafterboth
--- a/tests/src/eth/collectionLimits.test.ts
+++ b/tests/src/eth/collectionLimits.test.ts
@@ -1,6 +1,7 @@
 import {IKeyringPair} from '@polkadot/types/types';
 import {Pallets} from '../util';
 import {expect, itEth, usingEthPlaygrounds} from './util';
+import {CollectionLimits} from './util/playgrounds/types';
 
 
 describe('Can set collection limits', () => {
@@ -44,20 +45,33 @@
         transfersEnabled: false,
       };
      
-      const collection = helper.ethNativeContract.collection(collectionAddress, testCase.case, owner);
-      await collection.methods.setCollectionLimit('accountTokenOwnershipLimit', limits.accountTokenOwnershipLimit).send();
-      await collection.methods.setCollectionLimit('sponsoredDataSize', limits.sponsoredDataSize).send();
-      await collection.methods.setCollectionLimit('sponsoredDataRateLimit', limits.sponsoredDataRateLimit).send();
-      await collection.methods.setCollectionLimit('tokenLimit', limits.tokenLimit).send();
-      await collection.methods.setCollectionLimit('sponsorTransferTimeout', limits.sponsorTransferTimeout).send();
-      await collection.methods.setCollectionLimit('sponsorApproveTimeout', limits.sponsorApproveTimeout).send();
-      await collection.methods.setCollectionLimit('ownerCanTransfer', limits.ownerCanTransfer).send();
-      await collection.methods.setCollectionLimit('ownerCanDestroy', limits.ownerCanDestroy).send();
-      await collection.methods.setCollectionLimit('transfersEnabled', limits.transfersEnabled).send();
+      const collectionEvm = helper.ethNativeContract.collection(collectionAddress, testCase.case, owner);
+      await collectionEvm.methods.setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, limits.accountTokenOwnershipLimit).send();
+      await collectionEvm.methods.setCollectionLimit(CollectionLimits.SponsoredDataSize, true, limits.sponsoredDataSize).send();
+      await collectionEvm.methods.setCollectionLimit(CollectionLimits.SponsoredDataRateLimit, true, limits.sponsoredDataRateLimit).send();
+      await collectionEvm.methods.setCollectionLimit(CollectionLimits.TokenLimit, true, limits.tokenLimit).send();
+      await collectionEvm.methods.setCollectionLimit(CollectionLimits.SponsorTransferTimeout, true, limits.sponsorTransferTimeout).send();
+      await collectionEvm.methods.setCollectionLimit(CollectionLimits.SponsorApproveTimeout, true, limits.sponsorApproveTimeout).send();
+      await collectionEvm.methods.setCollectionLimit(CollectionLimits.OwnerCanTransfer, true, limits.ownerCanTransfer).send();
+      await collectionEvm.methods.setCollectionLimit(CollectionLimits.OwnerCanDestroy, true, limits.ownerCanDestroy).send();
+      await collectionEvm.methods.setCollectionLimit(CollectionLimits.TransferEnabled, true, limits.transfersEnabled).send();
       
+      // Check limits from sub:
       const data = (await helper.rft.getData(collectionId))!;
       expect(data.raw.limits).to.deep.eq(expectedLimits);
       expect(await helper.collection.getEffectiveLimits(collectionId)).to.deep.eq(expectedLimits);
+      // Check limits from eth:
+      const limitsEvm = await collectionEvm.methods.collectionLimits().call({from: owner});
+      expect(limitsEvm).to.have.length(9);
+      expect(limitsEvm[0]).to.deep.eq([CollectionLimits.AccountTokenOwnership.toString(), true, limits.accountTokenOwnershipLimit.toString()]);
+      expect(limitsEvm[1]).to.deep.eq([CollectionLimits.SponsoredDataSize.toString(), true, limits.sponsoredDataSize.toString()]);
+      expect(limitsEvm[2]).to.deep.eq([CollectionLimits.SponsoredDataRateLimit.toString(), true, limits.sponsoredDataRateLimit.toString()]);
+      expect(limitsEvm[3]).to.deep.eq([CollectionLimits.TokenLimit.toString(), true, limits.tokenLimit.toString()]);
+      expect(limitsEvm[4]).to.deep.eq([CollectionLimits.SponsorTransferTimeout.toString(), true, limits.sponsorTransferTimeout.toString()]);
+      expect(limitsEvm[5]).to.deep.eq([CollectionLimits.SponsorApproveTimeout.toString(), true, limits.sponsorApproveTimeout.toString()]);
+      expect(limitsEvm[6]).to.deep.eq([CollectionLimits.OwnerCanTransfer.toString(), true, limits.ownerCanTransfer.toString()]);
+      expect(limitsEvm[7]).to.deep.eq([CollectionLimits.OwnerCanDestroy.toString(), true, limits.ownerCanDestroy.toString()]);
+      expect(limitsEvm[8]).to.deep.eq([CollectionLimits.TransferEnabled.toString(), true, limits.transfersEnabled.toString()]);
     }));
 });
 
@@ -84,17 +98,48 @@
       const owner = await helper.eth.createAccountWithBalance(donor);
       const {collectionAddress} = await helper.eth.createCollection(testCase.case, owner, 'Limits', 'absolutely anything', 'ISNI', 18);
       const collectionEvm = helper.ethNativeContract.collection(collectionAddress, testCase.case, owner);
+
+      // Cannot set non-existing limit
       await expect(collectionEvm.methods
-        .setCollectionLimit('badLimit', '1')
-        .call()).to.be.rejectedWith('unknown limit "badLimit"');
-      
+        .setCollectionLimit(9, true, 1)
+        .call()).to.be.rejectedWith('Returned error: VM Exception while processing transaction: revert Value not convertible into enum "CollectionLimits"');      
+        
+      // Cannot disable limits
+      await expect(collectionEvm.methods
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, false, 200)
+        .call()).to.be.rejectedWith('Returned error: VM Exception while processing transaction: revert user can\'t disable limits');
+
       await expect(collectionEvm.methods
-        .setCollectionLimit(Object.keys(invalidLimits)[0], invalidLimits.accountTokenOwnershipLimit)
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, invalidLimits.accountTokenOwnershipLimit)
         .call()).to.be.rejectedWith(`can't convert value to u32 "${invalidLimits.accountTokenOwnershipLimit}"`);
-      
+ 
       await expect(collectionEvm.methods
-        .setCollectionLimit(Object.keys(invalidLimits)[1], invalidLimits.transfersEnabled)
+        .setCollectionLimit(CollectionLimits.TransferEnabled, true, 3)
         .call()).to.be.rejectedWith(`can't convert value to boolean "${invalidLimits.transfersEnabled}"`);
+
+      expect(() => collectionEvm.methods
+        .setCollectionLimit(CollectionLimits.SponsoredDataSize, true, -1).send()).to.throw('value out-of-bounds');
     }));
+
+  [
+    {case: 'nft' as const, requiredPallets: []},
+    {case: 'rft' as const, requiredPallets: [Pallets.ReFungible]},
+    {case: 'ft' as const, requiredPallets: []},
+  ].map(testCase =>
+    itEth.ifWithPallets(`Non-owner and non-admin cannot set collection limits for ${testCase.case}`, testCase.requiredPallets || [], async ({helper}) => {
+      const owner = await helper.eth.createAccountWithBalance(donor);
+      const nonOwner = await helper.eth.createAccountWithBalance(donor);
+      const {collectionAddress} = await helper.eth.createCollection(testCase.case, owner, 'Limits', 'absolutely anything', 'FLO', 18);
+
+      const collectionEvm = helper.ethNativeContract.collection(collectionAddress, testCase.case, owner);
+      await expect(collectionEvm.methods
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, 1000)
+        .call({from: nonOwner}))
+        .to.be.rejectedWith('NoPermission');
+
+      await expect(collectionEvm.methods
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, 1000)
+        .send({from: nonOwner}))
+        .to.be.rejected;
+    }));
 });
-  
\ No newline at end of file
modifiedtests/src/eth/createFTCollection.test.tsdiffbeforeafterboth
--- a/tests/src/eth/createFTCollection.test.ts
+++ b/tests/src/eth/createFTCollection.test.ts
@@ -18,6 +18,7 @@
 import {evmToAddress} from '@polkadot/util-crypto';
 import {Pallets, requirePalletsOrSkip} from '../util';
 import {expect, itEth, usingEthPlaygrounds} from './util';
+import { CollectionLimits } from './util/playgrounds/types';
 
 const DECIMALS = 18;
 
@@ -196,7 +197,7 @@
     }
     {
       await expect(peasantCollection.methods
-        .setCollectionLimit('accountTokenOwnershipLimit', '1000')
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, 1000)
         .call()).to.be.rejectedWith(EXPECTED_ERROR);
     }
   });
@@ -221,7 +222,7 @@
     }
     {
       await expect(peasantCollection.methods
-        .setCollectionLimit('accountTokenOwnershipLimit', '1000')
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, 1000)
         .call()).to.be.rejectedWith(EXPECTED_ERROR);
     }
   });    
modifiedtests/src/eth/createNFTCollection.test.tsdiffbeforeafterboth
--- a/tests/src/eth/createNFTCollection.test.ts
+++ b/tests/src/eth/createNFTCollection.test.ts
@@ -17,6 +17,7 @@
 import {evmToAddress} from '@polkadot/util-crypto';
 import {IKeyringPair} from '@polkadot/types/types';
 import {expect, itEth, usingEthPlaygrounds} from './util';
+import { CollectionLimits } from './util/playgrounds/types';
 
 
 describe('Create NFT collection from EVM', () => {
@@ -207,7 +208,7 @@
     }
     {
       await expect(malfeasantCollection.methods
-        .setCollectionLimit('accountTokenOwnershipLimit', '1000')
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, 1000)
         .call()).to.be.rejectedWith(EXPECTED_ERROR);
     }
   });
@@ -232,7 +233,7 @@
     }
     {
       await expect(malfeasantCollection.methods
-        .setCollectionLimit('accountTokenOwnershipLimit', '1000')
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, 1000)
         .call()).to.be.rejectedWith(EXPECTED_ERROR);
     }
   });
modifiedtests/src/eth/createRFTCollection.test.tsdiffbeforeafterboth
--- a/tests/src/eth/createRFTCollection.test.ts
+++ b/tests/src/eth/createRFTCollection.test.ts
@@ -18,6 +18,7 @@
 import {IKeyringPair} from '@polkadot/types/types';
 import {Pallets, requirePalletsOrSkip} from '../util';
 import {expect, itEth, usingEthPlaygrounds} from './util';
+import {CollectionLimits} from './util/playgrounds/types';
 
 
 describe('Create RFT collection from EVM', () => {
@@ -239,7 +240,7 @@
     }
     {
       await expect(peasantCollection.methods
-        .setCollectionLimit('accountTokenOwnershipLimit', '1000')
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, 1000)
         .call()).to.be.rejectedWith(EXPECTED_ERROR);
     }
   });
@@ -264,7 +265,7 @@
     }
     {
       await expect(peasantCollection.methods
-        .setCollectionLimit('accountTokenOwnershipLimit', '1000')
+        .setCollectionLimit(CollectionLimits.AccountTokenOwnership, true, 1000)
         .call()).to.be.rejectedWith(EXPECTED_ERROR);
     }
   });
modifiedtests/src/eth/events.test.tsdiffbeforeafterboth
--- a/tests/src/eth/events.test.ts
+++ b/tests/src/eth/events.test.ts
@@ -19,7 +19,7 @@
 import {EthUniqueHelper, itEth, usingEthPlaygrounds} from './util';
 import {IEvent, TCollectionMode} from '../util/playgrounds/types';
 import {Pallets, requirePalletsOrSkip} from '../util';
-import {EthTokenPermissions, NormalizedEvent} from './util/playgrounds/types';
+import {CollectionLimits, EthTokenPermissions, NormalizedEvent} from './util/playgrounds/types';
 
 let donor: IKeyringPair;
   
@@ -234,7 +234,7 @@
   });
   const {unsubscribe, collectedEvents: subEvents} = await helper.subscribeEvents([{section: 'common', names: ['CollectionLimitSet']}]);
   {
-    await collection.methods.setCollectionLimit('ownerCanTransfer', 0n).send({from: owner});
+    await collection.methods.setCollectionLimit(CollectionLimits.OwnerCanTransfer, true, 0).send({from: owner});
     await helper.wait.newBlocks(1);
     expect(ethEvents).to.be.like([
       {
modifiedtests/src/eth/util/playgrounds/types.tsdiffbeforeafterboth
--- a/tests/src/eth/util/playgrounds/types.ts
+++ b/tests/src/eth/util/playgrounds/types.ts
@@ -24,4 +24,15 @@
   Mutable,
   TokenOwner,
   CollectionAdmin
-}
\ No newline at end of file
+}
+export enum CollectionLimits {
+  AccountTokenOwnership,
+	SponsoredDataSize,
+	SponsoredDataRateLimit,
+	TokenLimit,
+	SponsorTransferTimeout,
+	SponsorApproveTimeout,
+	OwnerCanTransfer,
+	OwnerCanDestroy,
+	TransferEnabled
+}
modifiedtests/src/pallet-presence.test.tsdiffbeforeafterboth
--- a/tests/src/pallet-presence.test.ts
+++ b/tests/src/pallet-presence.test.ts
@@ -84,6 +84,7 @@
         );
       } else if (chain.eq('UNIQUE')) {
         // Insert Unique additional pallets here
+        requiredPallets.push(foreignAssets);
       }
     });
   });
--- a/pallets/nonfungible/src/stubs/UniqueNFT.sol
+++ b/pallets/nonfungible/src/stubs/UniqueNFT.sol
@@ -42,18 +42,19 @@
 	/// @param permissions Permissions for keys.
 	/// @dev EVM selector for this function is: 0xbd92983a,
 	///  or in textual repr: setTokenPropertyPermissions((string,(uint8,bool)[])[])
-	function setTokenPropertyPermissions(Tuple48[] memory permissions) public {
+	function setTokenPropertyPermissions(Tuple59[] memory permissions) public {
 		require(false, stub_error);
 		permissions;
 		dummy = 0;
 	}
 
+	/// @notice Get permissions for token properties.
 	/// @dev EVM selector for this function is: 0xf23d7790,
 	///  or in textual repr: tokenPropertyPermissions()
-	function tokenPropertyPermissions() public view returns (Tuple48[] memory) {
+	function tokenPropertyPermissions() public view returns (Tuple59[] memory) {
 		require(false, stub_error);
 		dummy;
-		return new Tuple48[](0);
+		return new Tuple59[](0);
 	}
 
 	// /// @notice Set token property value.
@@ -132,26 +133,30 @@
 	bytes value;
 }
 
+/// @dev Ethereum representation of TokenPermissions (see [`up_data_structs::PropertyPermission`]) fields as an enumeration.
 enum EthTokenPermissions {
+	/// @dev Permission to change the property and property permission. See [`up_data_structs::PropertyPermission::mutable`]
 	Mutable,
+	/// @dev Change permission for the collection administrator. See [`up_data_structs::PropertyPermission::token_owner`]
 	TokenOwner,
+	/// @dev Permission to change the property for the owner of the token. See [`up_data_structs::PropertyPermission::collection_admin`]
 	CollectionAdmin
 }
 
 /// @dev anonymous struct
-struct Tuple48 {
+struct Tuple59 {
 	string field_0;
-	Tuple46[] field_1;
+	Tuple57[] field_1;
 }
 
 /// @dev anonymous struct
-struct Tuple46 {
+struct Tuple57 {
 	EthTokenPermissions field_0;
 	bool field_1;
 }
 
 /// @title A contract that allows you to work with collections.
-/// @dev the ERC-165 identifier for this interface is 0xb5e1747f
+/// @dev the ERC-165 identifier for this interface is 0x81172a75
 contract Collection is Dummy, ERC165 {
 	// /// Set collection property.
 	// ///
@@ -291,6 +296,27 @@
 		return Tuple30(0x0000000000000000000000000000000000000000, 0);
 	}
 
+	/// Get current collection limits.
+	///
+	/// @return Array of tuples (byte, bool, uint256) with limits and their values. Order of limits:
+	/// 	"accountTokenOwnershipLimit",
+	/// 	"sponsoredDataSize",
+	/// 	"sponsoredDataRateLimit",
+	/// 	"tokenLimit",
+	/// 	"sponsorTransferTimeout",
+	/// 	"sponsorApproveTimeout"
+	///  	"ownerCanTransfer",
+	/// 	"ownerCanDestroy",
+	/// 	"transfersEnabled"
+	/// Return `false` if a limit not set.
+	/// @dev EVM selector for this function is: 0xf63bc572,
+	///  or in textual repr: collectionLimits()
+	function collectionLimits() public view returns (Tuple33[] memory) {
+		require(false, stub_error);
+		dummy;
+		return new Tuple33[](0);
+	}
+
 	/// Set limits for the collection.
 	/// @dev Throws error if limit not found.
 	/// @param limit Name of the limit. Valid names:
@@ -303,12 +329,18 @@
 	///  	"ownerCanTransfer",
 	/// 	"ownerCanDestroy",
 	/// 	"transfersEnabled"
+	/// @param status enable\disable limit. Works only with `true`.
 	/// @param value Value of the limit.
-	/// @dev EVM selector for this function is: 0x4ad890a8,
-	///  or in textual repr: setCollectionLimit(string,uint256)
-	function setCollectionLimit(string memory limit, uint256 value) public {
+	/// @dev EVM selector for this function is: 0x88150bd0,
+	///  or in textual repr: setCollectionLimit(uint8,bool,uint256)
+	function setCollectionLimit(
+		CollectionLimits limit,
+		bool status,
+		uint256 value
+	) public {
 		require(false, stub_error);
 		limit;
+		status;
 		value;
 		dummy = 0;
 	}
@@ -390,19 +422,19 @@
 	/// Returns nesting for a collection
 	/// @dev EVM selector for this function is: 0x22d25bfe,
 	///  or in textual repr: collectionNestingRestrictedCollectionIds()
-	function collectionNestingRestrictedCollectionIds() public view returns (Tuple34 memory) {
+	function collectionNestingRestrictedCollectionIds() public view returns (Tuple39 memory) {
 		require(false, stub_error);
 		dummy;
-		return Tuple34(false, new uint256[](0));
+		return Tuple39(false, new uint256[](0));
 	}
 
 	/// Returns permissions for a collection
 	/// @dev EVM selector for this function is: 0x5b2eaf4b,
 	///  or in textual repr: collectionNestingPermissions()
-	function collectionNestingPermissions() public view returns (Tuple37[] memory) {
+	function collectionNestingPermissions() public view returns (Tuple42[] memory) {
 		require(false, stub_error);
 		dummy;
-		return new Tuple37[](0);
+		return new Tuple42[](0);
 	}
 
 	/// Set the collection access method.
@@ -582,17 +614,46 @@
 }
 
 /// @dev anonymous struct
-struct Tuple37 {
+struct Tuple42 {
 	CollectionPermissions field_0;
 	bool field_1;
 }
 
 /// @dev anonymous struct
-struct Tuple34 {
+struct Tuple39 {
 	bool field_0;
 	uint256[] field_1;
 }
 
+/// @dev [`CollectionLimits`](up_data_structs::CollectionLimits) representation for EVM.
+enum CollectionLimits {
+	/// @dev How many tokens can a user have on one account.
+	AccountTokenOwnership,
+	/// @dev How many bytes of data are available for sponsorship.
+	SponsoredDataSize,
+	/// @dev In any case, chain default: [`SponsoringRateLimit::SponsoringDisabled`]
+	SponsoredDataRateLimit,
+	/// @dev How many tokens can be mined into this collection.
+	TokenLimit,
+	/// @dev Timeouts for transfer sponsoring.
+	SponsorTransferTimeout,
+	/// @dev Timeout for sponsoring an approval in passed blocks.
+	SponsorApproveTimeout,
+	/// @dev Whether the collection owner of the collection can send tokens (which belong to other users).
+	OwnerCanTransfer,
+	/// @dev Can the collection owner burn other people's tokens.
+	OwnerCanDestroy,
+	/// @dev Is it possible to send tokens from this collection between users.
+	TransferEnabled
+}
+
+/// @dev anonymous struct
+struct Tuple33 {
+	CollectionLimits field_0;
+	bool field_1;
+	uint256 field_2;
+}
+
 /// @dev anonymous struct
 struct Tuple30 {
 	address field_0;
modifiedpallets/refungible/src/stubs/UniqueRefungible.rawdiffbeforeafterboth

binary blob — no preview

modifiedpallets/refungible/src/stubs/UniqueRefungible.soldiffbeforeafterboth
--- a/pallets/refungible/src/stubs/UniqueRefungible.sol
+++ b/pallets/refungible/src/stubs/UniqueRefungible.sol
@@ -42,7 +42,7 @@
 	/// @param permissions Permissions for keys.
 	/// @dev EVM selector for this function is: 0xbd92983a,
 	///  or in textual repr: setTokenPropertyPermissions((string,(uint8,bool)[])[])
-	function setTokenPropertyPermissions(Tuple53[] memory permissions) public {
+	function setTokenPropertyPermissions(Tuple58[] memory permissions) public {
 		require(false, stub_error);
 		permissions;
 		dummy = 0;
@@ -51,10 +51,10 @@
 	/// @notice Get permissions for token properties.
 	/// @dev EVM selector for this function is: 0xf23d7790,
 	///  or in textual repr: tokenPropertyPermissions()
-	function tokenPropertyPermissions() public view returns (Tuple53[] memory) {
+	function tokenPropertyPermissions() public view returns (Tuple58[] memory) {
 		require(false, stub_error);
 		dummy;
-		return new Tuple53[](0);
+		return new Tuple58[](0);
 	}
 
 	// /// @notice Set token property value.
@@ -133,26 +133,30 @@
 	bytes value;
 }
 
+/// @dev Ethereum representation of TokenPermissions (see [`up_data_structs::PropertyPermission`]) fields as an enumeration.
 enum EthTokenPermissions {
+	/// @dev Permission to change the property and property permission. See [`up_data_structs::PropertyPermission::mutable`]
 	Mutable,
+	/// @dev Change permission for the collection administrator. See [`up_data_structs::PropertyPermission::token_owner`]
 	TokenOwner,
+	/// @dev Permission to change the property for the owner of the token. See [`up_data_structs::PropertyPermission::collection_admin`]
 	CollectionAdmin
 }
 
 /// @dev anonymous struct
-struct Tuple53 {
+struct Tuple58 {
 	string field_0;
-	Tuple51[] field_1;
+	Tuple56[] field_1;
 }
 
 /// @dev anonymous struct
-struct Tuple51 {
+struct Tuple56 {
 	EthTokenPermissions field_0;
 	bool field_1;
 }
 
 /// @title A contract that allows you to work with collections.
-/// @dev the ERC-165 identifier for this interface is 0xb5e1747f
+/// @dev the ERC-165 identifier for this interface is 0x81172a75
 contract Collection is Dummy, ERC165 {
 	// /// Set collection property.
 	// ///
@@ -292,6 +296,27 @@
 		return Tuple29(0x0000000000000000000000000000000000000000, 0);
 	}
 
+	/// Get current collection limits.
+	///
+	/// @return Array of tuples (byte, bool, uint256) with limits and their values. Order of limits:
+	/// 	"accountTokenOwnershipLimit",
+	/// 	"sponsoredDataSize",
+	/// 	"sponsoredDataRateLimit",
+	/// 	"tokenLimit",
+	/// 	"sponsorTransferTimeout",
+	/// 	"sponsorApproveTimeout"
+	///  	"ownerCanTransfer",
+	/// 	"ownerCanDestroy",
+	/// 	"transfersEnabled"
+	/// Return `false` if a limit not set.
+	/// @dev EVM selector for this function is: 0xf63bc572,
+	///  or in textual repr: collectionLimits()
+	function collectionLimits() public view returns (Tuple32[] memory) {
+		require(false, stub_error);
+		dummy;
+		return new Tuple32[](0);
+	}
+
 	/// Set limits for the collection.
 	/// @dev Throws error if limit not found.
 	/// @param limit Name of the limit. Valid names:
@@ -304,12 +329,18 @@
 	///  	"ownerCanTransfer",
 	/// 	"ownerCanDestroy",
 	/// 	"transfersEnabled"
+	/// @param status enable\disable limit. Works only with `true`.
 	/// @param value Value of the limit.
-	/// @dev EVM selector for this function is: 0x4ad890a8,
-	///  or in textual repr: setCollectionLimit(string,uint256)
-	function setCollectionLimit(string memory limit, uint256 value) public {
+	/// @dev EVM selector for this function is: 0x88150bd0,
+	///  or in textual repr: setCollectionLimit(uint8,bool,uint256)
+	function setCollectionLimit(
+		CollectionLimits limit,
+		bool status,
+		uint256 value
+	) public {
 		require(false, stub_error);
 		limit;
+		status;
 		value;
 		dummy = 0;
 	}
@@ -391,19 +422,19 @@
 	/// Returns nesting for a collection
 	/// @dev EVM selector for this function is: 0x22d25bfe,
 	///  or in textual repr: collectionNestingRestrictedCollectionIds()
-	function collectionNestingRestrictedCollectionIds() public view returns (Tuple33 memory) {
+	function collectionNestingRestrictedCollectionIds() public view returns (Tuple38 memory) {
 		require(false, stub_error);
 		dummy;
-		return Tuple33(false, new uint256[](0));
+		return Tuple38(false, new uint256[](0));
 	}
 
 	/// Returns permissions for a collection
 	/// @dev EVM selector for this function is: 0x5b2eaf4b,
 	///  or in textual repr: collectionNestingPermissions()
-	function collectionNestingPermissions() public view returns (Tuple36[] memory) {
+	function collectionNestingPermissions() public view returns (Tuple41[] memory) {
 		require(false, stub_error);
 		dummy;
-		return new Tuple36[](0);
+		return new Tuple41[](0);
 	}
 
 	/// Set the collection access method.
@@ -583,17 +614,46 @@
 }
 
 /// @dev anonymous struct
-struct Tuple36 {
+struct Tuple41 {
 	CollectionPermissions field_0;
 	bool field_1;
 }
 
 /// @dev anonymous struct
-struct Tuple33 {
+struct Tuple38 {
 	bool field_0;
 	uint256[] field_1;
 }
 
+/// @dev [`CollectionLimits`](up_data_structs::CollectionLimits) representation for EVM.
+enum CollectionLimits {
+	/// @dev How many tokens can a user have on one account.
+	AccountTokenOwnership,
+	/// @dev How many bytes of data are available for sponsorship.
+	SponsoredDataSize,
+	/// @dev In any case, chain default: [`SponsoringRateLimit::SponsoringDisabled`]
+	SponsoredDataRateLimit,
+	/// @dev How many tokens can be mined into this collection.
+	TokenLimit,
+	/// @dev Timeouts for transfer sponsoring.
+	SponsorTransferTimeout,
+	/// @dev Timeout for sponsoring an approval in passed blocks.
+	SponsorApproveTimeout,
+	/// @dev Whether the collection owner of the collection can send tokens (which belong to other users).
+	OwnerCanTransfer,
+	/// @dev Can the collection owner burn other people's tokens.
+	OwnerCanDestroy,
+	/// @dev Is it possible to send tokens from this collection between users.
+	TransferEnabled
+}
+
+/// @dev anonymous struct
+struct Tuple32 {
+	CollectionLimits field_0;
+	bool field_1;
+	uint256 field_2;
+}
+
 /// @dev anonymous struct
 struct Tuple29 {
 	address field_0;
modifiedtests/src/eth/abi/fungible.jsondiffbeforeafterboth
--- a/tests/src/eth/api/UniqueFungible.sol
+++ b/tests/src/eth/api/UniqueFungible.sol
@@ -13,7 +13,7 @@
 }
 
 /// @title A contract that allows you to work with collections.
-/// @dev the ERC-165 identifier for this interface is 0xb5e1747f
+/// @dev the ERC-165 identifier for this interface is 0x81172a75
 interface Collection is Dummy, ERC165 {
 	// /// Set collection property.
 	// ///
@@ -104,6 +104,23 @@
 	///  or in textual repr: collectionSponsor()
 	function collectionSponsor() external view returns (Tuple8 memory);
 
+	/// Get current collection limits.
+	///
+	/// @return Array of tuples (byte, bool, uint256) with limits and their values. Order of limits:
+	/// 	"accountTokenOwnershipLimit",
+	/// 	"sponsoredDataSize",
+	/// 	"sponsoredDataRateLimit",
+	/// 	"tokenLimit",
+	/// 	"sponsorTransferTimeout",
+	/// 	"sponsorApproveTimeout"
+	///  	"ownerCanTransfer",
+	/// 	"ownerCanDestroy",
+	/// 	"transfersEnabled"
+	/// Return `false` if a limit not set.
+	/// @dev EVM selector for this function is: 0xf63bc572,
+	///  or in textual repr: collectionLimits()
+	function collectionLimits() external view returns (Tuple19[] memory);
+
 	/// Set limits for the collection.
 	/// @dev Throws error if limit not found.
 	/// @param limit Name of the limit. Valid names:
@@ -116,10 +133,15 @@
 	///  	"ownerCanTransfer",
 	/// 	"ownerCanDestroy",
 	/// 	"transfersEnabled"
+	/// @param status enable\disable limit. Works only with `true`.
 	/// @param value Value of the limit.
-	/// @dev EVM selector for this function is: 0x4ad890a8,
-	///  or in textual repr: setCollectionLimit(string,uint256)
-	function setCollectionLimit(string memory limit, uint256 value) external;
+	/// @dev EVM selector for this function is: 0x88150bd0,
+	///  or in textual repr: setCollectionLimit(uint8,bool,uint256)
+	function setCollectionLimit(
+		CollectionLimits limit,
+		bool status,
+		uint256 value
+	) external;
 
 	/// Get contract address.
 	/// @dev EVM selector for this function is: 0xf6b4dfb4,
@@ -169,12 +191,12 @@
 	/// Returns nesting for a collection
 	/// @dev EVM selector for this function is: 0x22d25bfe,
 	///  or in textual repr: collectionNestingRestrictedCollectionIds()
-	function collectionNestingRestrictedCollectionIds() external view returns (Tuple20 memory);
+	function collectionNestingRestrictedCollectionIds() external view returns (Tuple24 memory);
 
 	/// Returns permissions for a collection
 	/// @dev EVM selector for this function is: 0x5b2eaf4b,
 	///  or in textual repr: collectionNestingPermissions()
-	function collectionNestingPermissions() external view returns (Tuple23[] memory);
+	function collectionNestingPermissions() external view returns (Tuple27[] memory);
 
 	/// Set the collection access method.
 	/// @param mode Access mode
@@ -289,7 +311,7 @@
 }
 
 /// @dev anonymous struct
-struct Tuple23 {
+struct Tuple27 {
 	CollectionPermissions field_0;
 	bool field_1;
 }
@@ -300,11 +322,40 @@
 }
 
 /// @dev anonymous struct
-struct Tuple20 {
+struct Tuple24 {
 	bool field_0;
 	uint256[] field_1;
 }
 
+/// @dev [`CollectionLimits`](up_data_structs::CollectionLimits) representation for EVM.
+enum CollectionLimits {
+	/// @dev How many tokens can a user have on one account.
+	AccountTokenOwnership,
+	/// @dev How many bytes of data are available for sponsorship.
+	SponsoredDataSize,
+	/// @dev In any case, chain default: [`SponsoringRateLimit::SponsoringDisabled`]
+	SponsoredDataRateLimit,
+	/// @dev How many tokens can be mined into this collection.
+	TokenLimit,
+	/// @dev Timeouts for transfer sponsoring.
+	SponsorTransferTimeout,
+	/// @dev Timeout for sponsoring an approval in passed blocks.
+	SponsorApproveTimeout,
+	/// @dev Whether the collection owner of the collection can send tokens (which belong to other users).
+	OwnerCanTransfer,
+	/// @dev Can the collection owner burn other people's tokens.
+	OwnerCanDestroy,
+	/// @dev Is it possible to send tokens from this collection between users.
+	TransferEnabled
+}
+
+/// @dev anonymous struct
+struct Tuple19 {
+	CollectionLimits field_0;
+	bool field_1;
+	uint256 field_2;
+}
+
 /// @dev Property struct
 struct Property {
 	string key;
modifiedtests/src/eth/api/UniqueNFT.soldiffbeforeafterboth
--- a/tests/src/eth/api/UniqueNFT.sol
+++ b/tests/src/eth/api/UniqueNFT.sol
@@ -30,11 +30,12 @@
 	/// @param permissions Permissions for keys.
 	/// @dev EVM selector for this function is: 0xbd92983a,
 	///  or in textual repr: setTokenPropertyPermissions((string,(uint8,bool)[])[])
-	function setTokenPropertyPermissions(Tuple43[] memory permissions) external;
+	function setTokenPropertyPermissions(Tuple52[] memory permissions) external;
 
+	/// @notice Get permissions for token properties.
 	/// @dev EVM selector for this function is: 0xf23d7790,
 	///  or in textual repr: tokenPropertyPermissions()
-	function tokenPropertyPermissions() external view returns (Tuple43[] memory);
+	function tokenPropertyPermissions() external view returns (Tuple52[] memory);
 
 	// /// @notice Set token property value.
 	// /// @dev Throws error if `msg.sender` has no permission to edit the property.
@@ -85,26 +86,30 @@
 	bytes value;
 }
 
+/// @dev Ethereum representation of TokenPermissions (see [`up_data_structs::PropertyPermission`]) fields as an enumeration.
 enum EthTokenPermissions {
+	/// @dev Permission to change the property and property permission. See [`up_data_structs::PropertyPermission::mutable`]
 	Mutable,
+	/// @dev Change permission for the collection administrator. See [`up_data_structs::PropertyPermission::token_owner`]
 	TokenOwner,
+	/// @dev Permission to change the property for the owner of the token. See [`up_data_structs::PropertyPermission::collection_admin`]
 	CollectionAdmin
 }
 
 /// @dev anonymous struct
-struct Tuple43 {
+struct Tuple52 {
 	string field_0;
-	Tuple41[] field_1;
+	Tuple50[] field_1;
 }
 
 /// @dev anonymous struct
-struct Tuple41 {
+struct Tuple50 {
 	EthTokenPermissions field_0;
 	bool field_1;
 }
 
 /// @title A contract that allows you to work with collections.
-/// @dev the ERC-165 identifier for this interface is 0xb5e1747f
+/// @dev the ERC-165 identifier for this interface is 0x81172a75
 interface Collection is Dummy, ERC165 {
 	// /// Set collection property.
 	// ///
@@ -195,6 +200,23 @@
 	///  or in textual repr: collectionSponsor()
 	function collectionSponsor() external view returns (Tuple27 memory);
 
+	/// Get current collection limits.
+	///
+	/// @return Array of tuples (byte, bool, uint256) with limits and their values. Order of limits:
+	/// 	"accountTokenOwnershipLimit",
+	/// 	"sponsoredDataSize",
+	/// 	"sponsoredDataRateLimit",
+	/// 	"tokenLimit",
+	/// 	"sponsorTransferTimeout",
+	/// 	"sponsorApproveTimeout"
+	///  	"ownerCanTransfer",
+	/// 	"ownerCanDestroy",
+	/// 	"transfersEnabled"
+	/// Return `false` if a limit not set.
+	/// @dev EVM selector for this function is: 0xf63bc572,
+	///  or in textual repr: collectionLimits()
+	function collectionLimits() external view returns (Tuple30[] memory);
+
 	/// Set limits for the collection.
 	/// @dev Throws error if limit not found.
 	/// @param limit Name of the limit. Valid names:
@@ -207,10 +229,15 @@
 	///  	"ownerCanTransfer",
 	/// 	"ownerCanDestroy",
 	/// 	"transfersEnabled"
+	/// @param status enable\disable limit. Works only with `true`.
 	/// @param value Value of the limit.
-	/// @dev EVM selector for this function is: 0x4ad890a8,
-	///  or in textual repr: setCollectionLimit(string,uint256)
-	function setCollectionLimit(string memory limit, uint256 value) external;
+	/// @dev EVM selector for this function is: 0x88150bd0,
+	///  or in textual repr: setCollectionLimit(uint8,bool,uint256)
+	function setCollectionLimit(
+		CollectionLimits limit,
+		bool status,
+		uint256 value
+	) external;
 
 	/// Get contract address.
 	/// @dev EVM selector for this function is: 0xf6b4dfb4,
@@ -260,12 +287,12 @@
 	/// Returns nesting for a collection
 	/// @dev EVM selector for this function is: 0x22d25bfe,
 	///  or in textual repr: collectionNestingRestrictedCollectionIds()
-	function collectionNestingRestrictedCollectionIds() external view returns (Tuple31 memory);
+	function collectionNestingRestrictedCollectionIds() external view returns (Tuple35 memory);
 
 	/// Returns permissions for a collection
 	/// @dev EVM selector for this function is: 0x5b2eaf4b,
 	///  or in textual repr: collectionNestingPermissions()
-	function collectionNestingPermissions() external view returns (Tuple34[] memory);
+	function collectionNestingPermissions() external view returns (Tuple38[] memory);
 
 	/// Set the collection access method.
 	/// @param mode Access mode
@@ -380,7 +407,7 @@
 }
 
 /// @dev anonymous struct
-struct Tuple34 {
+struct Tuple38 {
 	CollectionPermissions field_0;
 	bool field_1;
 }
@@ -391,11 +418,40 @@
 }
 
 /// @dev anonymous struct
-struct Tuple31 {
+struct Tuple35 {
 	bool field_0;
 	uint256[] field_1;
 }
 
+/// @dev [`CollectionLimits`](up_data_structs::CollectionLimits) representation for EVM.
+enum CollectionLimits {
+	/// @dev How many tokens can a user have on one account.
+	AccountTokenOwnership,
+	/// @dev How many bytes of data are available for sponsorship.
+	SponsoredDataSize,
+	/// @dev In any case, chain default: [`SponsoringRateLimit::SponsoringDisabled`]
+	SponsoredDataRateLimit,
+	/// @dev How many tokens can be mined into this collection.
+	TokenLimit,
+	/// @dev Timeouts for transfer sponsoring.
+	SponsorTransferTimeout,
+	/// @dev Timeout for sponsoring an approval in passed blocks.
+	SponsorApproveTimeout,
+	/// @dev Whether the collection owner of the collection can send tokens (which belong to other users).
+	OwnerCanTransfer,
+	/// @dev Can the collection owner burn other people's tokens.
+	OwnerCanDestroy,
+	/// @dev Is it possible to send tokens from this collection between users.
+	TransferEnabled
+}
+
+/// @dev anonymous struct
+struct Tuple30 {
+	CollectionLimits field_0;
+	bool field_1;
+	uint256 field_2;
+}
+
 /// @dev anonymous struct
 struct Tuple27 {
 	address field_0;
modifiedtests/src/eth/api/UniqueRefungible.soldiffbeforeafterboth
--- a/tests/src/eth/api/UniqueRefungible.sol
+++ b/tests/src/eth/api/UniqueRefungible.sol
@@ -30,12 +30,12 @@
 	/// @param permissions Permissions for keys.
 	/// @dev EVM selector for this function is: 0xbd92983a,
 	///  or in textual repr: setTokenPropertyPermissions((string,(uint8,bool)[])[])
-	function setTokenPropertyPermissions(Tuple47[] memory permissions) external;
+	function setTokenPropertyPermissions(Tuple51[] memory permissions) external;
 
 	/// @notice Get permissions for token properties.
 	/// @dev EVM selector for this function is: 0xf23d7790,
 	///  or in textual repr: tokenPropertyPermissions()
-	function tokenPropertyPermissions() external view returns (Tuple47[] memory);
+	function tokenPropertyPermissions() external view returns (Tuple51[] memory);
 
 	// /// @notice Set token property value.
 	// /// @dev Throws error if `msg.sender` has no permission to edit the property.
@@ -86,26 +86,30 @@
 	bytes value;
 }
 
+/// @dev Ethereum representation of TokenPermissions (see [`up_data_structs::PropertyPermission`]) fields as an enumeration.
 enum EthTokenPermissions {
+	/// @dev Permission to change the property and property permission. See [`up_data_structs::PropertyPermission::mutable`]
 	Mutable,
+	/// @dev Change permission for the collection administrator. See [`up_data_structs::PropertyPermission::token_owner`]
 	TokenOwner,
+	/// @dev Permission to change the property for the owner of the token. See [`up_data_structs::PropertyPermission::collection_admin`]
 	CollectionAdmin
 }
 
 /// @dev anonymous struct
-struct Tuple47 {
+struct Tuple51 {
 	string field_0;
-	Tuple45[] field_1;
+	Tuple49[] field_1;
 }
 
 /// @dev anonymous struct
-struct Tuple45 {
+struct Tuple49 {
 	EthTokenPermissions field_0;
 	bool field_1;
 }
 
 /// @title A contract that allows you to work with collections.
-/// @dev the ERC-165 identifier for this interface is 0xb5e1747f
+/// @dev the ERC-165 identifier for this interface is 0x81172a75
 interface Collection is Dummy, ERC165 {
 	// /// Set collection property.
 	// ///
@@ -196,6 +200,23 @@
 	///  or in textual repr: collectionSponsor()
 	function collectionSponsor() external view returns (Tuple26 memory);
 
+	/// Get current collection limits.
+	///
+	/// @return Array of tuples (byte, bool, uint256) with limits and their values. Order of limits:
+	/// 	"accountTokenOwnershipLimit",
+	/// 	"sponsoredDataSize",
+	/// 	"sponsoredDataRateLimit",
+	/// 	"tokenLimit",
+	/// 	"sponsorTransferTimeout",
+	/// 	"sponsorApproveTimeout"
+	///  	"ownerCanTransfer",
+	/// 	"ownerCanDestroy",
+	/// 	"transfersEnabled"
+	/// Return `false` if a limit not set.
+	/// @dev EVM selector for this function is: 0xf63bc572,
+	///  or in textual repr: collectionLimits()
+	function collectionLimits() external view returns (Tuple29[] memory);
+
 	/// Set limits for the collection.
 	/// @dev Throws error if limit not found.
 	/// @param limit Name of the limit. Valid names:
@@ -208,10 +229,15 @@
 	///  	"ownerCanTransfer",
 	/// 	"ownerCanDestroy",
 	/// 	"transfersEnabled"
+	/// @param status enable\disable limit. Works only with `true`.
 	/// @param value Value of the limit.
-	/// @dev EVM selector for this function is: 0x4ad890a8,
-	///  or in textual repr: setCollectionLimit(string,uint256)
-	function setCollectionLimit(string memory limit, uint256 value) external;
+	/// @dev EVM selector for this function is: 0x88150bd0,
+	///  or in textual repr: setCollectionLimit(uint8,bool,uint256)
+	function setCollectionLimit(
+		CollectionLimits limit,
+		bool status,
+		uint256 value
+	) external;
 
 	/// Get contract address.
 	/// @dev EVM selector for this function is: 0xf6b4dfb4,
@@ -261,12 +287,12 @@
 	/// Returns nesting for a collection
 	/// @dev EVM selector for this function is: 0x22d25bfe,
 	///  or in textual repr: collectionNestingRestrictedCollectionIds()
-	function collectionNestingRestrictedCollectionIds() external view returns (Tuple30 memory);
+	function collectionNestingRestrictedCollectionIds() external view returns (Tuple34 memory);
 
 	/// Returns permissions for a collection
 	/// @dev EVM selector for this function is: 0x5b2eaf4b,
 	///  or in textual repr: collectionNestingPermissions()
-	function collectionNestingPermissions() external view returns (Tuple33[] memory);
+	function collectionNestingPermissions() external view returns (Tuple37[] memory);
 
 	/// Set the collection access method.
 	/// @param mode Access mode
@@ -381,7 +407,7 @@
 }
 
 /// @dev anonymous struct
-struct Tuple33 {
+struct Tuple37 {
 	CollectionPermissions field_0;
 	bool field_1;
 }
@@ -392,11 +418,40 @@
 }
 
 /// @dev anonymous struct
-struct Tuple30 {
+struct Tuple34 {
 	bool field_0;
 	uint256[] field_1;
 }
 
+/// @dev [`CollectionLimits`](up_data_structs::CollectionLimits) representation for EVM.
+enum CollectionLimits {
+	/// @dev How many tokens can a user have on one account.
+	AccountTokenOwnership,
+	/// @dev How many bytes of data are available for sponsorship.
+	SponsoredDataSize,
+	/// @dev In any case, chain default: [`SponsoringRateLimit::SponsoringDisabled`]
+	SponsoredDataRateLimit,
+	/// @dev How many tokens can be mined into this collection.
+	TokenLimit,
+	/// @dev Timeouts for transfer sponsoring.
+	SponsorTransferTimeout,
+	/// @dev Timeout for sponsoring an approval in passed blocks.
+	SponsorApproveTimeout,
+	/// @dev Whether the collection owner of the collection can send tokens (which belong to other users).
+	OwnerCanTransfer,
+	/// @dev Can the collection owner burn other people's tokens.
+	OwnerCanDestroy,
+	/// @dev Is it possible to send tokens from this collection between users.
+	TransferEnabled
+}
+
+/// @dev anonymous struct
+struct Tuple29 {
+	CollectionLimits field_0;
+	bool field_1;
+	uint256 field_2;
+}
+
 /// @dev anonymous struct
 struct Tuple26 {
 	address field_0;
modifiedtests/src/eth/collectionLimits.test.tsdiffbeforeafterboth